// LoadString.cpp : Defines the entry point for the console application.
//

// This is based on Michael Kaplan's code here:
// http://blogs.msdn.com/michkap/archive/2007/01/18/1487464.aspx
// (His C# program does something slightly different.)

#include "stdafx.h"

int _tmain(int argc, _TCHAR* argv[])
{
	if (argc != 3)
	{
#ifdef _WIN64
		wprintf(L"\nLoadString.exe (64-bit version)\n\n");
#else
		wprintf(L"\nLoadString.exe (32-bit version)\n\n");
#endif

		wprintf(L"This tool will print a string resource from a DLL file.\n"
				L"For example, if you see something like @%%SystemRoot%%\\system32\\shell32.dll,-9227 in the registry\n"
				L"and want to know what that string actually is you can use this tool to find out.\n"
				L"\n"
				L"Usage:\n"
				L" LoadString.exe <path to dll> <resource id>\n"
				L"\n"
				L"Examples:\n"
				L" > LoadString.exe %%SystemRoot%%\\system32\\shell32.dll -9227\n"
				L" String resource = \"Documents\"\n"
				L"\n"
				L" > LoadString.exe %%SystemRoot%%\\system32\\shell32.dll -30598\n"
				L" String resource = \"PNG Image\"\n"
				L"\n"
				L" > LoadString.exe \"%%ProgramFiles%%\\Windows Mail\\msoeres.dll\" -7496\n"
				L" String resource = \"Email Previewer\"\n"
				L"\n"
				L"Updates and C++ source:\n"
				L" http://www.pretentiousname.com\n\n");
	}
	else
	{
		const wchar_t *szDllPath = argv[1];
		const wchar_t *szNumber = argv[2];

		int iNumber = _wtoi(szNumber);

		if (iNumber < 0)
		{
			// ID numbers are negative in the registry for some reason.
			iNumber = -iNumber;
		}

		if (iNumber == 0 && szNumber[0] != L'0')
		{
#ifdef _WIN64
			wprintf(L"[64-bit] Failed to convert \"%s\" into a number.", szNumber);
#else
			wprintf(L"[32-bit] Failed to convert \"%s\" into a number.", szNumber);
#endif
		}
		else
		{
			HMODULE hModule = ::LoadLibraryEx(szDllPath, NULL, LOAD_LIBRARY_AS_DATAFILE);

			if (hModule == NULL)
			{
				DWORD dwErr = GetLastError();

#ifdef _WIN64
				wprintf(L"[64-bit] Failed to load \"%s\": Error %lu\n", szDllPath, dwErr);
#else
				wprintf(L"[32-bit] Failed to load \"%s\": Error %lu\n", szDllPath, dwErr);
#endif
			}
			else
			{
				wchar_t szBuffer[1024];

				if (0 >= ::LoadString(hModule, iNumber, szBuffer, sizeof(szBuffer)/sizeof(szBuffer[0])))
				{
					DWORD dwErr = GetLastError();

#ifdef _WIN64
					wprintf(L"[64-bit] Failed to load string resources: Error %lu\n", dwErr);
#else
					wprintf(L"[32-bit] Failed to load string resources: Error %lu\n", dwErr);
#endif
				}
				else
				{
					wprintf(L"String resource = \"%s\"\n", szBuffer);
				}

				FreeLibrary(hModule);
			}
		}
	}

	return 0;
}

